#!/usr/bin/perl -T

# Copyright  1997 UC Regents. All rights reserved.
# Copyleft  1999,2000 Michiel Dethmers. All rights reversed.
# Document maintained at http://www.quipu.co.uk/SWISH-E/AutoSwish/.
# Author: Michiel Dethmers, michiel@quipu.co.uk
# Version 1.0 21/01/2000. 
#
# Originally Based on swish_create.pl by Unknown Author from Berkeley University
# The Name AutoSwish was identified with that script, but they have stopped 
# distribution because of lack of time to update it to the new swish-e 
# configuration. (swish-e version 1.3).
# If anyone wants to dispute the use of AutoSwish as a name, please get in touch
# as soon as possible.
#
# This program and library is free software; you can redistribute it and/or
# modify it under the terms of the GNU (Library) General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#* GNU (Library) General Public License for more details.
#
# You should have received a copy of the GNU (Library) General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#---------------------------------------------------------
#
# Purpose: AutoSwish is a perl script that will manage your Swish-e Indexes via
# a set of Forms and Pages. It allows you to specify your indexes and re-index them
# You can use any search script with the indexes that are generated, but
# with this script you find Swish-Search.cgi, which is a basic search script
# using the same environment as AutoSwish.

#- Values you may want to change are in config.pl -----------------------------#

# Check config.pl for Configuration variables

#- Specify variables to secure the environment --------------------------------#

$ENV{BASH_ENV} = ""; # for running from command line
$ENV{'SCRIPT_FILENAME'} =~ /(.*)\/.*/;
$SCRIPT_ROOT = $1;
$ENV{'PATH'} = "/usr/local/bin:/usr/bin:$SCRIPT_ROOT";

#- Main Program --------------------------------------------------------------#

# untaint the user, if authentication is used 
if ($ENV{REMOTE_USER}) {
  $ENV{REMOTE_USER} =~ /(\w+)/;
  $userid = $1;
} else {
  $userid = "Unauthenticated User";
}
  
use CGI qw(:cgi-lib);
ReadParse;

require "$SCRIPT_ROOT/../config.pl"; # read the configuration variables
require "$SCRIPT_ROOT/../lib.pl";

# list of fields allowed
$fields = '(PropertyNames|WordCharacters|BeginCharacters|EndCharacters|todo|IndexComments|IndexName|IndexDescription|UseStemming|IgnoreFirstChar|IgnoreLastChar|MaxWordLimit|MinWordLimit|IndexAdmin|BeginCharacters|EndCharacters|WordCharacters|IndexDir|IndexOnly|NoContents|FollowSymLinks|ReplaceRules|ReplaceRules2|ReplaceRules3|FileRules|FileRules2|FileRules3|FileRules4|IgnoreLimit|IgnoreWords|todo|SimpleFlag|ConfigFile|MetaNames|RemoveFlag|RemoveRoot|ReIndex|force)';

foreach $key (keys %in) {
   if ($key !~ /\b$fields\b/o) {
      &return_error (500, "AutoSwish Error",
                          "Invalid Information in Form.");
  } 
} 

if ($in{'SimpleFlag'} =~ /y/) {
  # the Quick form is used, specify some settings
   $in{IndexDir} =~ s/\/$//;
   $in{IndexDir} =~ s/^\///;
   $in{IndexDir} =~ /([\w\/-_]+)/;
	 $path = $1 || &web_die("Invalid File path");
	 $in{'ReplaceRules'} = 'replace "'.$WEBDIR.'" "http://'. $SERVERNAME .'"';
   $fileroot = $path;
   $fileroot =~ s/.*\///;
   $conf = "$fileroot.conf";
   $in{IndexDir} = "$WEBDIR/$path";
   $in{IndexName} = $fileroot;
   $in{IndexDescription} = "All Documents in $path";
   &web_die("<h3>Cannot find Directory, $path</h3>") if (!-e "$WEBDIR/$path");
} else {
   if ($in{'ConfigFile'} =~ /\.conf/) {
      $fileroot = $`;
			$conf = "$fileroot.conf";
	 } elsif ($in{'todo'} eq "createconfig") {
			# make sure we have a valid configfile if we are creating a new one
			$i++ while (-e "$CONFIGDIR/SwishConfig$i.conf");
			$conf = "SwishConfig$i.conf";
			$conf .= ".conf" if $conf !~ /\.conf$/;
	 		$conf =~ /\.conf/;
	 		$fileroot = $`;
   } else {
      &return_error ("CGI ERROR",
                     "Error: No Configuration file specified.");
   }
}

if ($in{'todo'} =~ /^list(.*)/) {
   &listindex("$1");
} elsif ($in{'todo'} =~ /reindex/) {
   &reindex;
} elsif ($in{'todo'} =~ /remove/) {
   &remove;
} elsif ($in{'todo'} =~ /createconfig/) {
   &createconfig;
} elsif ($in{'todo'} =~ /index/) {
   &index;
} elsif ($in{'todo'} =~ /edit/) {
   &edit;
} else {
   &return_error (500, "CGI Error",
                       "Form Data Corrupted.");
}

#- Subroutines ---------------------------------------------------------------#

# Subroutine for listing the existing configuration files with some command.

sub listindex {
	local($command) = $_[0] || "reindex";
	$msg{remove} = "Choose a configuration file to remove<BR>\nAll associated files will also be deleted";
	$msg{reindex} = "Choose a configuration file to begin re-indexing:";
	$msg{edit} = "Choose a configuration file to edit:";
	$msg2{reindex} = "Remember that indexing takes time. Be patient!";
	$msg2{remove} = "<B><U>Caution</U> once deleted these files cannot be recovered!</B>";

	print &html_header("$msg{$command}");
	print "<CENTER>\n";
	print "$msg{$command}\n";
	print "<TABLE>\n<TR><TD>\n\n";
  print &ListConfigFiles($command);
	print "</TABLE>$msg2{$command}\n</CENTER>\n";
	print &html_footer;
}

# Subroutine for removing indexes and their associated files.
sub remove {

	@deletelist = ("$CONFIGDIR/$fileroot.conf", "$INDEXDIR/$fileroot.swish");
	unlink (@deletelist);
	
	print &html_header("Removing indexes with SWISH-E: List of deleted files");
	print <<EOT;
	<CENTER>
	<table><td>
	<B>The following files have been removed:</B>
	<UL>
EOT
	foreach (@deletelist) {
		print "<li>$_";
		(-e $_) ? print " ... <font color=red>Failed!</font><br>" : print " ... Successful!<br>";
	}
	print <<EOT;
	</UL></td></table>
	</CENTER>
EOT
	print &html_footer;
}

# Subroutine for creating a confguration file.
sub createconfig {

   mkdir ("$CONFIGDIR", 0777);
   open (OUTFILE, ">$CONFIGDIR/$conf") || &web_die("Cannot create $CONFIGDIR/$conf");

   $fields = '(PropertyNames|SimpleFlag|todo|ConfigFile|IndexDir|IndexOnly|NoContents|MetaNames|FollowSymLinks|ReplaceRules|ReplaceRules2|ReplaceRules3|FileRules|FileRules2|FileRules3|FileRules4|IgnoreLimit|IgnoreWords)';
   $fields = '(PropertyNames|todo|IndexComments|IndexName|IndexDescription|UseStemming|IgnoreFirstChar|IgnoreLastChar|MaxWordLimit|MinWordLimit|IndexAdmin|BeginCharacters|EndCharacters|WordCharacters|IndexDir|IndexOnly|NoContents|FollowSymLinks|ReplaceRules|ReplaceRules2|ReplaceRules3|FileRules|FileRules2|FileRules3|FileRules4|IgnoreLimit|IgnoreWords|todo|SimpleFlag|ConfigFile|MetaNames|RemoveFlag|RemoveRoot|ReIndex)';
   # subset of above, which are the CGI control variables and should not be written to file
   $nofilefields = '(SimpleFlag|ConfigFile|todo)';
   # subset of above, which need to be commented when empty, swish-e chokes on them otherwise
   $comment_if_empty = '(PropertyNames|IndexAdmin|MaxWordLimit|BeginCharacters|IgnoreFirstChar|IgnoreLastChar|EndCharacters|IndexName|IndexDescription|WordCharacters|FileRules|MinWordLimit)';

   my ($d,$m,$y,$dow) = (localtime)[3,4,5,6];
   $y += 1900;

   print OUTFILE "\n# This SWISH-E configuration file was created on $DoW[$dow], $d $Months[$m] $y";
   print OUTFILE "\n# by $userid from $ENV{'REMOTE_HOST'} ($ENV{'REMOTE_ADDR'})\n\n";

   foreach $key (keys %in) {
      if  ($key !~ /\b$fields\b/o) {
         &return_error (500, "CGI Error",
                             "Invalid Field Name Used.");
      } else {
         $keyname = $key;
         $keyname  =~ s/\d$//;
				 next if $keyname =~ /\b$nofilefields\b/o;
				 $keyname = '# ' . $keyname if ($in{$key} =~ /^\s*$/ && $keyname =~ /\b$comment_if_empty\b/o);
         print OUTFILE "$keyname $in{$key}\n";
      }
   }

   print OUTFILE "IndexFile $INDEXDIR/$fileroot.swish\n";
   print OUTFILE "IndexReport 1\n";

   close (OUTFILE) || &web_die("Error writing config file: $!");
   print &html_header("Indexing with SWISH-E: Step 2, indexing");
   print <<End_of_INDEXPAGE;

<H2 ALIGN=CENTER>Step 2: File Indexing</H2>
<P ALIGN=CENTER>
The configuration file <B>$conf</B> has been successfully created.<BR><BR>
The next step is to index your files.<BR>
<B>Be Patient.</B> Depending on the size and number of files to be indexed, this could take quite a long time<BR>
(as much as 1 sec. per file).<BR><BR>
<CENTER>
<FORM ACTION="$ENV{'SCRIPT_NAME'}" METHOD=POST>
<INPUT TYPE="hidden" NAME="todo" VALUE="reindex">
<INPUT TYPE="hidden" NAME="ConfigFile" VALUE="$conf">
<INPUT TYPE="hidden" NAME="IndexDir" VALUE="$in{'IndexDir'}">
<INPUT TYPE="hidden" NAME="MetaNames" VALUE="$in{'MetaNames'}">
<INPUT TYPE="submit" VALUE="Index Now">
</FORM>
</CENTER>

End_of_INDEXPAGE

   print &html_footer;

}

# Subroutine to index or reindex a file

sub reindex {
  # call swish and output its data
  my $apache_version = $1;
  $| = 1; # flushing output only works with Apache 1.3 and up, before that the script needs to be nph-*
  if ($ENV{SERVER_SOFTWARE} =~ /Apache\/([\d\.]+)/) {
    $apache_version = $1;
  } else {
    $apache_version = "0.0";
  }
  
  # check for duplicating work
  my $modified = (stat(&getconfigitem("IndexFile","$CONFIGDIR/$conf")))[9];
  if (time - $modified < 10 * 60 * 60 && !$in{force}) { # 10 minutes
    print &html_header("Very Recently Indexed");
    print <<EOT;
<H3 ALIGN=CENTER>This Configuration has been indexed less than 10 minutes ago</H3>    
<p ALIGN=CENTER>It is unlikely it needs to be indexed again. Are you sure you want to index this one again?
<a href="$AUTOSWISH?ConfigFile=$conf&todo=reindex&force=yes">Yes</a></p>
EOT
    print "Choose another configuration file?:<BR>\n";
    print &ListConfigFiles("reindex");
    print &html_footer;
    exit;
  }
   
  if (eval ("$apacheversion < 1.3")) {
    print "Content-type: text/html\n";
    # opt out to an nph version
    print "Location: http://$SERVERNAME$CGIURL/SwishCreate/nph-index.pl?ConfigFile=$conf\n\n";
    exit;
  }

  print &html_header("Reindexing with SWISH-E: Processing Please Wait");
  print "<blockquote>";
  open(SWISH, "$SWISHDIR/swish-e -v 3 -c $CONFIGDIR/$conf|");
  while (<SWISH>) {
	  print "$_<br>\n";
  }
  print "</blockquote><H2 ALIGN=CENTER>Your Files Were Succesfully Re-Indexed</H2>";
  print "Command: $SWISHDIR/swish-e -c $CONFIGDIR/$conf<br>\n";
  print "Index file: $CONFIGDIR/$conf<br>\n";
  print "<a href=\"$CGIURL/Swish-Search.cgi\">Go to the Search Page</a><br>\n";
  print "Choose another configuration file?:<BR>\n";
  print &ListConfigFiles("reindex");
  print &html_footer;
}  

# Subroutine to edit a configuration
# We take the complete.shtml file and parse if for the data in our configfile

sub edit {
	# allow editing of configuration files
	# load the current configuraiton file
	open (CONF, "$CONFIGDIR/$conf");
  my %config;
	while (<CONF>) {
		next if /^#/;
		next if /^\s*$/;
		/([^\s]*) (.*)/;
		if (!$config{$1}) {
			$config{$1} = $2;
		} else {
			$i = 2;
			$i++ while $config{"$1$i"};
			$config{"$1$i"} = $2;
		}
	}
	close(CONF);
	open (TMPL, "$TEMPLATEDIR/$FULLTMPL") || &web_die("Can't find template $TEMPLATEDIR/$FULLTMPL") ;
	my $tmpl = join(//,<TMPL>);
	close(TMPL);

	# "uncheck" all radiobuttons
	$tmpl =~ s/<([^>]*?)checked([^>]*?)>/<\1\2>/ig;
	foreach $key (keys(%config)) {
		$config{$key} = &webify($config{$key});
		# note: this code only works on text and radio input, awful code, needs rejigging.

    $tmpl =~ s/type="text"[(\s|\w)*]?name="$key"(.*?)value=".*"/type="text"\1 name="$key"\2 value="$config{$key}"/i;		
        # avoid strange error when WordCharacters contain lots of gunk
		next if $tmpl !~ /type="radio"([^>]*)name="$key"(.*?)"/i;
		$tmpl =~ s/type="radio"([^>]*)name="$key"(.*?)value="$config{$key}"/type="radio"\1name="$key"\2value="$config{$key}" checked/i;
	}
	
	$tmpl =~ s/name="ConfigFile"(.*?)value="[^"]*"/name="ConfigFile"\1value="$in{'ConfigFile'}"/i;
	
	$tmpl =~ s/<title>([^<]*)<\/title>//i;
	print &html_header($1);
	$tmpl =~ s/<body [^>]+>//i;
	
	print $tmpl;
	print &html_footer;
	exit;
}
